/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package consoleengcalculator;

import engcalculator.data.Interval;
import engcalculator.data.IntervalList;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Polygon;
import java.util.LinkedList;
import javax.swing.JPanel;

/**
 *
 * @author simone
 */
public class FunctionDrawerJPanel extends JPanel {
    private LinkedList<Drawing> drawings = null;
    private CartesianPlane cartesianPlane = null;

    public void setCartesianPlane(CartesianPlane cartesianPlane) {
        this.cartesianPlane = cartesianPlane;
    }

    public CartesianPlane getCartesianPlane() {
        return cartesianPlane;
    }


    public void setDrawings(LinkedList<Drawing> drawings) {
        this.drawings = drawings;
    }

    @Override
    public void paint(Graphics g) {
        if (cartesianPlane == null) return;
        setBackground(ConsoleEngCalculatorDrawBox.colorConverter(cartesianPlane.getBackgroundCI()));
        super.paint(g);

        cartesianPlane.setPanelHeightRatio(getSize().getHeight());
        cartesianPlane.setPanelWidthRatio(getSize().getWidth());

        if (cartesianPlane.isShowGrid()) {

            g.setColor(ConsoleEngCalculatorDrawBox.colorConverter(cartesianPlane.getGridCI()));

            for (double x = Math.floor(cartesianPlane.getXMin()/cartesianPlane.getXTick())*cartesianPlane.getXTick(); x < cartesianPlane.getXMax(); x += cartesianPlane.getXTick()) {
                int xt = cartesianPlane.transformX(x);
                g.drawLine(xt,cartesianPlane.transformY(cartesianPlane.getYMin()),xt,cartesianPlane.transformY(cartesianPlane.getYMax()));
            }

            for (double y= Math.floor(cartesianPlane.getYMin()/cartesianPlane.getYTick())*cartesianPlane.getYTick(); y < cartesianPlane.getYMax(); y += cartesianPlane.getYTick()) {
                int yt = cartesianPlane.transformY(y);
                g.drawLine(cartesianPlane.transformX(cartesianPlane.getXMin()),yt,cartesianPlane.transformX(cartesianPlane.getXMax()),yt);
            }
        }

        if (cartesianPlane.isShowAxes()) {
            g.setColor(ConsoleEngCalculatorDrawBox.colorConverter(cartesianPlane.getAxesCI()));

            int zero = cartesianPlane.transformY(0);
            g.drawLine(cartesianPlane.transformX(cartesianPlane.getXMin()), zero,cartesianPlane.transformX(cartesianPlane.getXMax()), zero);

            zero = cartesianPlane.transformX(0);
            g.drawLine(zero,cartesianPlane.transformY(cartesianPlane.getYMin()),zero, cartesianPlane.transformY(cartesianPlane.getYMax()));

        }

        if (drawings == null) return;

        IntervalList data=null;
        int columns;
        for (Drawing d : drawings) {

            if (! d.isVisible()) continue;

            final boolean isColorSetByVal = ConsoleEngCalculatorDrawBox.isColorSetByVal(d.getColorIndex());
            final boolean isColorSetByTol = ConsoleEngCalculatorDrawBox.isColorSetByTol(d.getColorIndex());
            final boolean isVariableColor = isColorSetByTol || isColorSetByVal;

            data = d.getData().getValue();
            columns = data.getColumns();

            if (isVariableColor && columns != 3) continue;

            if (! isVariableColor) g.setColor(ConsoleEngCalculatorDrawBox.colorConverter(d.getColorIndex()));

            Interval x, y, xp = null, yp = null;
            Polygon p;
            double[] points = new double[8];
            boolean firstPoint = true;

            if (d.isJoined()) {
                for (int i=0; i<data.size(); i+=columns) {
                    x = data.get(i);
                    y = data.get(i+1);
                    if (! x.isProper() || ! y.isProper()) continue;
                    if (firstPoint) {
                        firstPoint = false;
                    } else {
                        try {
                            if (isColorSetByVal) g.setColor(Color.getHSBColor(cartesianPlane.transformC(data.get(i+2).getVal()), 1, 1));
                            else if (isColorSetByTol) g.setColor(Color.getHSBColor(cartesianPlane.transformC(data.get(i+2).getTol()), 1, 1));
                            g.drawRect(cartesianPlane.transformX(x.getLeft()), cartesianPlane.transformY(y.getRight()), cartesianPlane.zoomX(x.getRange()), cartesianPlane.zoomY(y.getRange()));
                            g.fillRect(cartesianPlane.transformX(x.getLeft()), cartesianPlane.transformY(y.getRight()), cartesianPlane.zoomX(x.getRange()), cartesianPlane.zoomY(y.getRange()));

                            if ((yp.getVal() < y.getVal() && xp.getVal() < x.getVal()) || (yp.getVal() >= y.getVal() && xp.getVal() >= x.getVal())) {//next point is higher-right or lower-left
                                    points[0] = xp.getLeft();
                                    points[1] = yp.getRight();
                                    points[2] = x.getLeft();
                                    points[3] = y.getRight();
                                    points[4] = x.getRight();
                                    points[5] = y.getLeft();
                                    points[6] = xp.getRight();
                                    points[7] = yp.getLeft();
                            } else {//next point is on the lower-right or higher-left
                                    points[0] = xp.getRight();
                                    points[1] = yp.getRight();
                                    points[2] = x.getRight();
                                    points[3] = y.getRight();
                                    points[4] = x.getLeft();
                                    points[5] = y.getLeft();
                                    points[6] = xp.getLeft();
                                    points[7] = yp.getLeft();
                            }

                            p = new Polygon ();
                            for (int j=0;j<7;++j)
                                p.addPoint(cartesianPlane.transformX(points[j]), cartesianPlane.transformY(points[++j]));

                            g.drawPolygon(p);
                            g.fillPolygon(p);
                        } catch (Exception e) {
                            //skip
                        }
                    }
                    xp = x;
                    yp = y;
                }
            } else {
                for (int i=0; i<data.size(); i += columns) {
                    x = data.get(i);
                    y = data.get(i+1);
                    try {
                        if (isColorSetByVal) g.setColor(Color.getHSBColor(cartesianPlane.transformC(data.get(i+2).getVal()), 1, 1));
                        else if (isColorSetByTol) g.setColor(Color.getHSBColor(cartesianPlane.transformC(data.get(i+2).getTol()), 1, 1));
                        g.drawRect(cartesianPlane.transformX(x.getLeft()), cartesianPlane.transformY(y.getRight()), cartesianPlane.zoomX(x.getRange()), cartesianPlane.zoomY(y.getRange()));
                        g.fillRect(cartesianPlane.transformX(x.getLeft()), cartesianPlane.transformY(y.getRight()), cartesianPlane.zoomX(x.getRange()), cartesianPlane.zoomY(y.getRange()));
                    } catch (Exception e) {
                        //skip
                    }
                }
            }
        }
    }


}
