/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package engcalculator.expression;


import engcalculator.expression.token.Token;
import engcalculator.data.Interval;
import engcalculator.data.Interval.WorkFlow;
import engcalculator.data.IntervalList;
import engcalculator.data.MatrixIntervalList;
import engcalculator.function.CalculusException;
import engcalculator.function.prefix.PrefixFunction;
import engcalculator.function.infix.InfixFunction;
import java.util.LinkedList;

/**
 *
 * @author simone
 */
public class Interpeter {
    private final static boolean DEBUG = false;

    private TokenList tokenizedExpression;
    private boolean keepRunning, repeatingNext;

    private boolean equals (IntervalList a, IntervalList b) {
        if (a.size() != b.size()) return false;
        for (int i=0; i < a.size(); ++i)
            try {
                if (a.get(i).getLeft() != b.get(i).getLeft() || a.get(i).getRight() != b.get(i).getRight()) return false;
            } catch (Exception e) {
                return false;
            }
        return true;
    }

    private boolean isInterval(IntervalList il1) {
        for (Interval i : il1)
            if (! i.isScalar()) return true;
        return false;
    }

    public LinkedList<String> listOfDuplicatedIntervals() {
        LinkedList<Token> result = new LinkedList<Token>();
        for (int i = 0; i < tokenizedExpression.size(); ++i) {
            Token t = tokenizedExpression.get(i);
            IntervalList il1;
            try {
                il1 = t.getIntervalList();
            } catch (Exception e) {
                continue;
            }
            if (il1 != null && isInterval(il1) && ! result.contains(t)) {
                int j;
                for (j=i+1; j < tokenizedExpression.size(); ++j) {
                    IntervalList il2;
                    try {
                        il2 = tokenizedExpression.get(j).getIntervalList();
                    } catch (Exception e) {
                        continue;
                    }
                    if (il2 != null && equals (il1, il2)) {
                        result.add(t);
                        break;
                    }
                }
            }
        }
        LinkedList<String> stringResult = new LinkedList<String>();
        for (Token t: result)
            stringResult.add(t.getSymbol());
       return stringResult;
    }

    //Numeric Interval and literal evaluator.
    private IntervalList interval() throws Exception {
        IntervalList i = tokenizedExpression.get().getIntervalList();
        if (i == null) i = new IntervalList ();//throw new ParsingException ("It was expected an interval or literal at "+tokenizedExpression.notUnderstoodToString());
        else tokenizedExpression.nextToken();
        return i;
    }

    // Brackets solver.
    private IntervalList squareBracket() throws Exception {
        if (! tokenizedExpression.isInList()) return new IntervalList ();

        if( "[".equals(tokenizedExpression.get().getTerminal()) ) {
            IntervalList answer;

            tokenizedExpression.nextToken();
            answer = computeInfixFunctionAtPriority(0);

            if (! (tokenizedExpression.isInList() )|| ! "]".equals (tokenizedExpression.get().getTerminal() ) ) throw new ParsingException ("The brackets does not match at "+tokenizedExpression.notUnderstoodToString());
            tokenizedExpression.nextToken();

            return new MatrixIntervalList (answer,answer.size());
        } else {
            return interval();
        }
    }

    // Brackets solver.
    private IntervalList roundBracket() throws Exception {
        if (! tokenizedExpression.isInList()) return new IntervalList ();

        if( "(".equals(tokenizedExpression.get().getTerminal()) ) {
            IntervalList answer;

            tokenizedExpression.nextToken();
            answer = computeInfixFunctionAtPriority(0);

            if (! (tokenizedExpression.isInList() )|| ! ")".equals (tokenizedExpression.get().getTerminal() ) ) throw new ParsingException ("The brackets does not match at "+tokenizedExpression.notUnderstoodToString());
            tokenizedExpression.nextToken();

            return answer;
        } else {
            return squareBracket();
        }
    }

    //Compute prefix function
    private IntervalList computePrefixFunction () throws Exception {
        IntervalList answer = null;
        PrefixFunction fun;

        if(tokenizedExpression.isInList() && (fun = tokenizedExpression.get().getPrefixFunction()) != null) {
            tokenizedExpression.nextToken();
            answer = computePrefixFunction(fun, computePrefixFunction () );
        } else  {
            answer = roundBracket ();
        }
        return answer;
    }

    //Compute infix function of given priority
    private IntervalList computeInfixFunctionAtPriority (int priority) throws Exception {
        IntervalList accumulator;

        if (priority > InfixFunction.getMaxPriority()) {
            accumulator = computePrefixFunction ();
        } else {
            accumulator = computeInfixFunctionAtPriority(priority+1);
            while( tokenizedExpression.isInList() ) {
                InfixFunction fun = tokenizedExpression.get().getInfixFunction();

                if (fun != null && fun.getPriority() == priority) {
                    tokenizedExpression.nextToken();
                    IntervalList rightOperand;
                    if (InfixFunction.isLeftAssociative(priority)) rightOperand = computeInfixFunctionAtPriority (priority+1);
                    else rightOperand = computeInfixFunctionAtPriority (priority);
                    accumulator = computeInfixFunction (fun, accumulator, rightOperand);
                } else break;
            }
        }
        return accumulator;
    }

    // subExpression solver.
    private IntervalList subExpression () throws Exception {
        boolean firstLoop = true;
        repeatingNext = false;
        IntervalList answer=null;
        Interval flowAnswer;

        while (tokenizedExpression.isInList() && keepRunning) {
            if (firstLoop) firstLoop = false;
            else Tokenizer.retokenize(tokenizedExpression);

            answer = computeInfixFunctionAtPriority (0);
            
            flowAnswer = findWorkFlow(answer);

            if (tokenizedExpression.isInList())
                if (";".equals(tokenizedExpression.get().getTerminal())) answer = null; //by default after a ';' there is not any answer
                else break; //if I am in the list but not at ';' there is something not understood

            changeWorkFlow(flowAnswer);
        }

        return answer;
    }

    public IntervalList parse() throws Exception {
        if (DEBUG) {
            System.out.println("Parsing tokenized expression: "+tokenizedExpression.toString());
        }

        tokenizedExpression.reset();

        IntervalList result;
        keepRunning = true;
        try  {
            result = subExpression ();
        } catch (CalculusException ce) {
            throw new CalculusException (tokenizedExpression.notUnderstoodToString()+ce.getMessage());
        }
        if (tokenizedExpression.isInList()) throw new ParsingException ("It was not possible to evaluate the ending part of expression. The partial result is: "+result);
        return result;
    }

    public Interpeter(TokenList tokenizedExpression) {
        this.tokenizedExpression = tokenizedExpression;
    }

    public void stopCalculus () {
        keepRunning = false;
    }

    protected IntervalList computePrefixFunction(PrefixFunction function, IntervalList argument) throws Exception {
        return function.compute( argument);
    }

    protected IntervalList computeInfixFunction(InfixFunction function, IntervalList argument1, IntervalList argument2) throws Exception {
        return function.compute(argument1, argument2);
    }
    
    protected Interval findWorkFlow (IntervalList answer) throws Exception {
        Interval flowAnswer = null;
        if (answer != null)
            for (Interval ans : answer) {
                boolean found = false;
                if (ans.getWorkFlow() != WorkFlow.NEXT_SUBEX)
                    if (found) throw new ParsingException ("It was found more than one flow control instruction in the same subexpression.") ;
                    else {
                        found = true;
                        flowAnswer = ans;
                    }
            }
        return flowAnswer;
    }
    
    protected void changeWorkFlow (Interval flowAnswer) throws Exception {
        if (repeatingNext) {
            tokenizedExpression.previousSubExpression();
            repeatingNext = false;
        } else if (flowAnswer != null) {
            switch (flowAnswer.getWorkFlow()) {
                case NEXT_SUBEX_THEN_THIS_SUBEX:
                    repeatingNext = true;
                    tokenizedExpression.nextToken();
                    break;
                case NEXT_SUBEX:
                    tokenizedExpression.nextToken();
                    break;
                case SKIP_NEXT_SUBEX:
                    tokenizedExpression.nextToken();
                    tokenizedExpression.nextSubExpression();
                    break;
                case PREVIOUS_SUBEX:
                    tokenizedExpression.previousSubExpression();
                    break;
                case THIS_SUBEX:
                    tokenizedExpression.thisSubExpression();
                    break;
                case STOP:
                    tokenizedExpression.goToEnd();
                    break;
                default:
                    throw new RuntimeException ("Internal error trying to find the proper control path in Parser.subExpression.");
            }
        } else tokenizedExpression.nextToken();
    }

}
